body {{ 
    color: #fff;
    background-color: #232629;
}}
::-webkit-scrollbar {{
    background: #232629;
}}
::-webkit-scrollbar-thumb {{
    background: {QTMATERIAL_SECONDARYLIGHTCOLOR};
}}
::-webkit-scrollbar-thumb:hover {{
    background: {QTMATERIAL_PRIMARYLIGHTCOLOR};
}}
::-webkit-scrollbar-corner {{
    background: #232629;
}}
p {{ 
    color: #fff;
}}
a {{
    color: #b5c3cd !important;
}}
a:hover {{
    color: #b8cad7 !important;
}}
.msg-user {{
    background: #303030;
    color: #fff;
}}
.msg-user p {{
    color: #fff;
}}
.msg-bot {{
    color: #fff;
}}
.msg-bot p {{
    color: #fff;
}}
.cmd {{
    color: #4d4d4d !important;
}}
.ts {{
    color: #4d4d4d;
}}
.list {{
}}
code {{
    color: {QTMATERIAL_PRIMARYLIGHTCOLOR};
}}
.code-wrapper ::-webkit-scrollbar {{
    background: #303030;
}}
.code-header-wrapper {{
    background-color: #303030;
}}
.code-header-action {{
    color: silver !important;
}}
.code-header-action:hover {{
    color: #fff;
}}
.code-header-action:hover img {{
    filter: brightness(130%);
}}
.code-header-lang {{
    color: #686868;
}}
.code-wrapper .code-header-action.code-header-copy:hover, 
.code-wrapper .code-header-action.code-header-collapse:hover {{
    transform: scale(1.06);
    background: var(--copy-btn-bg-hover, rgb(0 0 0 / 76%));
    border-color: var(--copy-btn-border, rgba(0, 0, 0, .08));
}}
.action-icons a:hover {{
    filter: brightness(130%);
}}
.name-bot::before {{
    background-color: {QTMATERIAL_PRIMARYLIGHTCOLOR};
}}
.name-user::before {{
    background-color: silver;
}}
.tool-output .content {{
    color: #4d4d4d !important;
}}

/* loader */
.lds-ring {{
   color: #fff;
}}
.append_live {{
  color: gray;
}}

.tips p {{
 color: #5e5e5e;
}}
.msg-box .name-header.name-bot {{
    color: #cfcfcf;
}}