* {{ 
  font-family: 'Lato' !important; 
  selection-background-color: #3390ff;
  selection-color: #fff;
}}
QWidget {{
  color: #000;
  margin: 1px;
  background-color: #eeeeee;
}}

QListView, 
QTreeView {{
  color: #000;
  background-color: #fff;
  border: 1px solid #c0c0c0;
}}
QListView::item:selected, 
QTreeView::item:selected {{
  color: #fff;
  selection-color: #000000;
}}
QListView::item:selected:focus, 
QTreeView::item:selected:focus {{
  color: #fff;
  selection-color: #000;
}}
QTabBar {{
  text-transform: none;
}}
QTabBar::tab:selected {{
  color: #000 !important;
}}
QTabBar::tab:top:selected,
QTabBar::tab:top:hover {{
  color: #000;
}}
QPushButton {{
  color: #3e3e3e;
  text-transform: none;
  background-color: {QTMATERIAL_SECONDARYCOLOR};
  border: 1px solid #fff;
}}
QLineEdit {{
  color: #000000; 
  border: 1px solid #c0c0c0;
  background-color: #fff;
}}
QLineEdit::focus {{
  color: #000000; 
  border: 1px solid #c0c0c0;
  background-color: #fff;
}}
QTextEdit {{
  color: #000000; 
  border: 1px solid #c0c0c0;
  background-color: #fff;
}}
QMenu {{ 
  background-color: #fff; 
  border: none;
}} 
QMenu::item {{ 
  color: #0e0e0e;
}} 
QComboBox,
QComboBox::focus {{
  background-color: #fff;
  color: #000000;
}}

QMainWindow::separator:horizontal,
QSplitter::handle:vertical {{
  width: 100%;
  height: 1px;
  background-color: #d4d4d4;
}}
QTableView::item,
QTreeView::item,
QListView::item {{
  color: #0e0e0e;
  padding: 4px;
  min-height: 20px;
}}

/* Subtle hover for list/tree items (non-selected) */
QListView::item:hover:!selected,
QTreeView::item:hover:!selected {{
  background-color: #f6f8fb;
}}

QCalendarWidget QWidget {{
  background: #fff;
}}
QScrollBar::handle {{
  background: {QTMATERIAL_PRIMARYCOLOR};
}}
QTreeView::item:disabled,
QListView::item:disabled,
QComboBox::item:disabled {{
  color: #828282;
  font-weight: bold
}}
QComboBox::item {{
  color: #000;
}}
QCheckBox::indicator,
QRadioButton::indicator {{
    border: 1px solid #828282 !important;
    width: 16px;
    height: 16px;
}}
QRadioButton::indicator,
QRadioButton::indicator::focus {{
  border-radius: 8px;
}}

QCheckBox::indicator:checked,
QGroupBox::indicator:checked {{
  image: none;
  background-color: #c1c2c2;
  selection-background-color: #fff; 
}}

QCheckBox::indicator:unchecked,
QGroupBox::indicator:unchecked {{
  image: none;
  background-color: #fff;
  border: 1px solid #828282 !important;
}}

QRadioButton::indicator:checked {{
  image: none;
  background-color: #c1c2c2;
  selection-background-color: #fff; 
  border-radius: 8px;
}}
QRadioButton::indicator:unchecked {{
  image: none;
  border: 1px solid #828282 !important;
  background-color: #fff;
}}
QHeaderView::section {{
  text-transform: none;
}}

QMenu::indicator {{
  border: none !important;
  width: 16px;
  height: 16px;
  image: none;
}}
QMenu::indicator:non-exclusive:unchecked {{
  image: none;
  background-color: #fff;
}}
QMenu::indicator:non-exclusive:unchecked:selected {{
  image: none;
  background-color: #fff;
}}
QMenu::indicator:non-exclusive:checked {{
  image: none;
  background-color: #c1c2c2;
}}
QMenu::indicator:non-exclusive:checked:selected {{
  image: none;
  background-color: #c1c2c2;
}}
QMenu::indicator:exclusive:unchecked {{
  image: none;
  background-color: #fff;
}}
QMenu::indicator:exclusive:unchecked:selected {{
  image: none;
  background-color: #fff;
}}
QMenu::indicator:exclusive:checked {{
  image: none;
  background-color: #c1c2c2;
}}
QMenu::indicator:exclusive:checked:selected {{
  image: none;
  background-color: #c1c2c2;
}}
QPushButton {{
  text-transform: none;
}}

.label-title {{
  font-weight: bold;
}}
.label-help,
.label-desc,
.label-chat-status {{
  color: #5d5d5d;
}}
.interpreter-input {{
  font-family: 'Monaspace Neon';
}}
.interpreter-output,
.code-editor {{
  font-family: 'Monaspace Neon';
  background: #fff; 
  color: #000;
}}

/* radius */
QPushButton,
QTextEdit,
QListView,
QTreeView,
QComboBox,
QLineEdit,
QWebView {{
  border-radius: 10px
}}
QMenu,
.layout-input {{
  border-top-right-radius: 10px;
  border-top-left-radius: 0px;
  border-bottom-right-radius: 10px;
  border-bottom-left-radius: 10px;
}}
.layout-search {{
  height: 35px;
}}
.layout-output-web {{
  border-radius: 10px;
}}
QListView,
QTreeView,
QTextEdit,
QLineEdit {{
 border: none;
}}
QComboBox::item:checked {{
  font-weight: bold;
}}

QScrollBar:vertical {{
  width: 6px;
}}
QCalendarWidget QToolButton,
QCalendarWidget QToolButton:hover {{
    border: none;
}}
.file-explorer QTreeView,
.file-explorer QHeaderView::section,
.file-explorer QTreeView::branch,
.file-explorer QTreeView::branch:open {{
  background: #fff;
}}
.file-explorer QHeaderView::section {{
  padding-top: 6px;
  padding-bottom: 2px;
  height: 20px;
}} 
QMenu::indicator {{
    border: 1px solid gray;
}}
QCalendarWidget QAbstractItemView::item:selected:focus,
QCalendarWidget QAbstractItemView::item:selected {{
    background: #ffffd4;
}}
QCalendarWidget QAbstractItemView::item:hover {{
    background: #f6f6f6;
}}
.file-explorer QTreeView::branch {{
  background: #fff;
}}

/* Node editor */
NodeEditor {{
  qproperty-gridBackColor: #ffffff;
  qproperty-gridPenColor: #eaeaea;

  qproperty-nodeBackgroundColor: #2d2f34;
  qproperty-nodeBorderColor: #4b4f57;
  qproperty-nodeSelectionColor: #ff9900;
  qproperty-nodeTitleColor: #3a3d44;

  qproperty-portInputColor: #66b2ff;
  qproperty-portOutputColor: #70e070;
  qproperty-portConnectedColor: #ffd166;

  qproperty-edgeColor: #c0c0c0;
  qproperty-edgeSelectedColor: #ff8a5c;
}}

/* Status Bar */
#StatusBarTimer {{
  color: #5d5d5d !important;
}}

/* ComboBox */
QComboBox QAbstractItemView {{
  border: 0px;
  border-radius: 10px;
  outline: 0;
}}

QListView#ComboPopupList {{
  border-radius: 10px;
  outline: 0;
  border: 0px solid #d6d6d6;
}}

QComboBox[name="SearchableCombo"] QAbstractItemView::item,
QListView#ComboPopupList::item {{
  min-height: 20px;
  padding: 8px 32px;
  margin: 0;
  color: #000;
}}

QComboBox QAbstractItemView::item:selected,
QListView#ComboPopupList::item:selected {{
   background: #fff;
   color: #000;
   selection-color: #000;
   font-weight: bold;
 }}

QListView#ComboPopupList::item:disabled {{
  color: #3f3f3f;
  background: #d9d9d9;
  padding: 15px;
}}

QComboBox QAbstractItemView::item:hover,
QListView#ComboPopupList::item:hover {{ }}
QWidget#ComboPopupWindow {{
  background: red;
  border: 0;
}}
QListView#ComboPopupList > QWidget#ComboPopupViewport {{
  background: red;
  border-radius: 10px;
}}