#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2025.08.23 21:00:00                  #
# ================================================== #

AGENT_MODE_ASSISTANT = "agent.mode.assistant"  # LlamaIndex
AGENT_MODE_OPENAI = "agent.mode.openai"  # openai-agent
AGENT_MODE_PLAN = "agent.mode.plan"  # LlamaIndex
AGENT_MODE_STEP = "agent.mode.step"  # LlamaIndex
AGENT_MODE_WORKFLOW = "agent.mode.workflow"  # LlamaIndex

AGENT_TYPE_LLAMA = "agent.type.llama"  # LlamaIndex
AGENT_TYPE_OPENAI = "agent.type.openai"  # OpenAI Agent
